@@ -1,27 +1,34 @@
 # -*- coding: utf-8 -*-
-
-# Copyright (C) 2019 Chris Caron <lead2gold@gmail.com>
-# All rights reserved.
+# BSD 3-Clause License
 #
-# This code is licensed under the MIT License.
+# Apprise - Push Notification Library.
+# Copyright (c) 2023, Chris Caron <lead2gold@gmail.com>
 #
-# Permission is hereby granted, free of charge, to any person obtaining a copy
-# of this software and associated documentation files(the "Software"), to deal
-# in the Software without restriction, including without limitation the rights
-# to use, copy, modify, merge, publish, distribute, sublicense, and / or sell
-# copies of the Software, and to permit persons to whom the Software is
-# furnished to do so, subject to the following conditions :
+# Redistribution and use in source and binary forms, with or without
+# modification, are permitted provided that the following conditions are met:
 #
-# The above copyright notice and this permission notice shall be included in
-# all copies or substantial portions of the Software.
+# 1. Redistributions of source code must retain the above copyright notice,
+#    this list of conditions and the following disclaimer.
 #
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.IN NO EVENT SHALL THE
-# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
-# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
-# THE SOFTWARE.
+# 2. Redistributions in binary form must reproduce the above copyright notice,
+#    this list of conditions and the following disclaimer in the documentation
+#    and/or other materials provided with the distribution.
+#
+# 3. Neither the name of the copyright holder nor the names of its
+#    contributors may be used to endorse or promote products derived from
+#    this software without specific prior written permission.
+#
+# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
+# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+# POSSIBILITY OF SUCH DAMAGE.
 
 import click
 import logging
@@ -73,28 +80,64 @@ DEFAULT_CONFIG_PATHS = (
     '~/.apprise/apprise.yml',
     '~/.config/apprise/apprise',
     '~/.config/apprise/apprise.yml',
+
+    # Global Configuration Support
+    '/etc/apprise',
+    '/etc/apprise.yml',
+    '/etc/apprise/apprise',
+    '/etc/apprise/apprise.yml',
 )
 
 # Define our paths to search for plugins
 DEFAULT_PLUGIN_PATHS = (
     '~/.apprise/plugins',
     '~/.config/apprise/plugins',
+
+    # Global Plugin Support
+    '/var/lib/apprise/plugins',
 )
 
 # Detect Windows
 if platform.system() == 'Windows':
     # Default Config Search Path for Windows Users
     DEFAULT_CONFIG_PATHS = (
-        expandvars('%APPDATA%/Apprise/apprise'),
-        expandvars('%APPDATA%/Apprise/apprise.yml'),
-        expandvars('%LOCALAPPDATA%/Apprise/apprise'),
-        expandvars('%LOCALAPPDATA%/Apprise/apprise.yml'),
+        expandvars('%APPDATA%\\Apprise\\apprise'),
+        expandvars('%APPDATA%\\Apprise\\apprise.yml'),
+        expandvars('%LOCALAPPDATA%\\Apprise\\apprise'),
+        expandvars('%LOCALAPPDATA%\\Apprise\\apprise.yml'),
+
+        #
+        # Global Support
+        #
+
+        # C:\ProgramData\Apprise\
+        expandvars('%ALLUSERSPROFILE%\\Apprise\\apprise'),
+        expandvars('%ALLUSERSPROFILE%\\Apprise\\apprise.yml'),
+
+        # C:\Program Files\Apprise
+        expandvars('%PROGRAMFILES%\\Apprise\\apprise'),
+        expandvars('%PROGRAMFILES%\\Apprise\\apprise.yml'),
+
+        # C:\Program Files\Common Files
+        expandvars('%COMMONPROGRAMFILES%\\Apprise\\apprise'),
+        expandvars('%COMMONPROGRAMFILES%\\Apprise\\apprise.yml'),
     )
 
     # Default Plugin Search Path for Windows Users
     DEFAULT_PLUGIN_PATHS = (
-        expandvars('%APPDATA%/Apprise/plugins'),
-        expandvars('%LOCALAPPDATA%/Apprise/plugins'),
+        expandvars('%APPDATA%\\Apprise\\plugins'),
+        expandvars('%LOCALAPPDATA%\\Apprise\\plugins'),
+
+        #
+        # Global Support
+        #
+
+        # C:\ProgramData\Apprise\plugins
+        expandvars('%ALLUSERSPROFILE%\\Apprise\\plugins'),
+        # C:\Program Files\Apprise\plugins
+        expandvars('%PROGRAMFILES%\\Apprise\\plugins'),
+        # C:\Program Files\Common Files
+        expandvars('%COMMONPROGRAMFILES%\\Apprise\\plugins'),
     )
 
 
