@@ -1,31 +1,39 @@
 # -*- coding: utf-8 -*-
+# BSD 3-Clause License
 #
-# Copyright (C) 2021 Chris Caron <lead2gold@gmail.com>
-# All rights reserved.
+# Apprise - Push Notification Library.
+# Copyright (c) 2023, Chris Caron <lead2gold@gmail.com>
 #
-# This code is licensed under the MIT License.
+# Redistribution and use in source and binary forms, with or without
+# modification, are permitted provided that the following conditions are met:
 #
-# Permission is hereby granted, free of charge, to any person obtaining a copy
-# of this software and associated documentation files(the "Software"), to deal
-# in the Software without restriction, including without limitation the rights
-# to use, copy, modify, merge, publish, distribute, sublicense, and / or sell
-# copies of the Software, and to permit persons to whom the Software is
-# furnished to do so, subject to the following conditions :
+# 1. Redistributions of source code must retain the above copyright notice,
+#    this list of conditions and the following disclaimer.
 #
-# The above copyright notice and this permission notice shall be included in
-# all copies or substantial portions of the Software.
+# 2. Redistributions in binary form must reproduce the above copyright notice,
+#    this list of conditions and the following disclaimer in the documentation
+#    and/or other materials provided with the distribution.
 #
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.IN NO EVENT SHALL THE
-# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
-# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
-# THE SOFTWARE.
+# 3. Neither the name of the copyright holder nor the names of its
+#    contributors may be used to endorse or promote products derived from
+#    this software without specific prior written permission.
+#
+# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
+# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+# POSSIBILITY OF SUCH DAMAGE.
 
 import re
 import requests
 from json import dumps
+import base64
 
 from .NotifyBase import NotifyBase
 from ..URLBase import PrivacyMode
@@ -35,6 +43,20 @@ from ..utils import validate_regex
 from ..AppriseLocale import gettext_lazy as _
 
 
+class AppriseAPIMethod:
+    """
+    Defines the method to post data tot he remote server
+    """
+    JSON = 'json'
+    FORM = 'form'
+
+
+APPRISE_API_METHODS = (
+    AppriseAPIMethod.FORM,
+    AppriseAPIMethod.JSON,
+)
+
+
 class NotifyAppriseAPI(NotifyBase):
     """
     A wrapper for Apprise (Persistent) API Notifications
@@ -57,7 +79,7 @@ class NotifyAppriseAPI(NotifyBase):
 
     # Depending on the number of transactions/notifications taking place, this
     # could take a while. 30 seconds should be enough to perform the task
-    socket_connect_timeout = 30.0
+    socket_read_timeout = 30.0
 
     # Disable throttle rate for Apprise API requests since they are normally
     # local anyway
@@ -112,6 +134,12 @@ class NotifyAppriseAPI(NotifyBase):
             'name': _('Tags'),
             'type': 'string',
         },
+        'method': {
+            'name': _('Query Method'),
+            'type': 'choice:string',
+            'values': APPRISE_API_METHODS,
+            'default': APPRISE_API_METHODS[0],
+        },
         'to': {
             'alias_of': 'token',
         },
@@ -125,7 +153,8 @@ class NotifyAppriseAPI(NotifyBase):
         },
     }
 
-    def __init__(self, token=None, tags=None, headers=None, **kwargs):
+    def __init__(self, token=None, tags=None, method=None, headers=None,
+                 **kwargs):
         """
         Initialize Apprise API Object
 
@@ -147,6 +176,14 @@ class NotifyAppriseAPI(NotifyBase):
             self.logger.warning(msg)
             raise TypeError(msg)
 
+        self.method = self.template_args['method']['default'] \
+            if not isinstance(method, str) else method.lower()
+
+        if self.method not in APPRISE_API_METHODS:
+            msg = 'The method specified ({}) is invalid.'.format(method)
+            self.logger.warning(msg)
+            raise TypeError(msg)
+
         # Build list of tags
         self.__tags = parse_list(tags)
 
@@ -162,8 +199,13 @@ class NotifyAppriseAPI(NotifyBase):
         Returns the URL built dynamically based on specified arguments.
         """
 
-        # Our URL parameters
-        params = self.url_parameters(privacy=privacy, *args, **kwargs)
+        # Define any URL parameters
+        params = {
+            'method': self.method,
+        }
+
+        # Extend our parameters
+        params.update(self.url_parameters(privacy=privacy, *args, **kwargs))
 
         # Append our headers into our parameters
         params.update({'+{}'.format(k): v for k, v in self.headers.items()})
@@ -202,15 +244,61 @@ class NotifyAppriseAPI(NotifyBase):
                    token=self.pprint(self.token, privacy, safe=''),
                    params=NotifyAppriseAPI.urlencode(params))
 
-    def send(self, body, title='', notify_type=NotifyType.INFO, **kwargs):
+    def send(self, body, title='', notify_type=NotifyType.INFO, attach=None,
+             **kwargs):
         """
         Perform Apprise API Notification
         """
 
-        headers = {}
+        # Prepare HTTP Headers
+        headers = {
+            'User-Agent': self.app_id,
+        }
+
         # Apply any/all header over-rides defined
         headers.update(self.headers)
 
+        attachments = []
+        files = []
+        if attach:
+            for no, attachment in enumerate(attach, start=1):
+                # Perform some simple error checking
+                if not attachment:
+                    # We could not access the attachment
+                    self.logger.error(
+                        'Could not access attachment {}.'.format(
+                            attachment.url(privacy=True)))
+                    return False
+
+                try:
+                    if self.method == AppriseAPIMethod.JSON:
+                        with open(attachment.path, 'rb') as f:
+                            # Output must be in a DataURL format (that's what
+                            # PushSafer calls it):
+                            attachments.append({
+                                'filename': attachment.name,
+                                'base64': base64.b64encode(f.read())
+                                .decode('utf-8'),
+                                'mimetype': attachment.mimetype,
+                            })
+
+                    else:  # AppriseAPIMethod.FORM
+                        files.append((
+                            'file{:02d}'.format(no),
+                            (
+                                attachment.name,
+                                open(attachment.path, 'rb'),
+                                attachment.mimetype,
+                            )
+                        ))
+
+                except (OSError, IOError) as e:
+                    self.logger.warning(
+                        'An I/O error occurred while reading {}.'.format(
+                            attachment.name if attachment else 'attachment'))
+                    self.logger.debug('I/O Exception: %s' % str(e))
+                    return False
+
         # prepare Apprise API Object
         payload = {
             # Apprise API Payload
@@ -220,6 +308,11 @@ class NotifyAppriseAPI(NotifyBase):
             'format': self.notify_format,
         }
 
+        if self.method == AppriseAPIMethod.JSON:
+            headers['Content-Type'] = 'application/json'
+            payload['attachments'] = attachments
+            payload = dumps(payload)
+
         if self.__tags:
             payload['tag'] = self.__tags
 
@@ -240,8 +333,8 @@ class NotifyAppriseAPI(NotifyBase):
 
         # Some entries can not be over-ridden
         headers.update({
-            'User-Agent': self.app_id,
-            'Content-Type': 'application/json',
+            # Our response to be in JSON format always
+            'Accept': 'application/json',
             # Pass our Source UUID4 Identifier
             'X-Apprise-ID': self.asset._uid,
             # Pass our current recursion count to our upstream server
@@ -259,9 +352,10 @@ class NotifyAppriseAPI(NotifyBase):
         try:
             r = requests.post(
                 url,
-                data=dumps(payload),
+                data=payload,
                 headers=headers,
                 auth=auth,
+                files=files if files else None,
                 verify=self.verify_certificate,
                 timeout=self.request_timeout,
             )
@@ -283,7 +377,8 @@ class NotifyAppriseAPI(NotifyBase):
                 return False
 
             else:
-                self.logger.info('Sent Apprise API notification.')
+                self.logger.info(
+                    'Sent Apprise API notification; method=%s.', self.method)
 
         except requests.RequestException as e:
             self.logger.warning(
@@ -294,6 +389,18 @@ class NotifyAppriseAPI(NotifyBase):
             # Return; we're done
             return False
 
+        except (OSError, IOError) as e:
+            self.logger.warning(
+                'An I/O error occurred while reading one of the '
+                'attached files.')
+            self.logger.debug('I/O Exception: %s' % str(e))
+            return False
+
+        finally:
+            for file in files:
+                # Ensure all files are closed
+                file[1][1].close()
+
         return True
 
     @staticmethod
@@ -370,4 +477,9 @@ class NotifyAppriseAPI(NotifyBase):
                 # re-assemble our full path
                 results['fullpath'] = '/'.join(entries)
 
+        # Set method if specified
+        if 'method' in results['qsd'] and len(results['qsd']['method']):
+            results['method'] = \
+                NotifyAppriseAPI.unquote(results['qsd']['method'])
+
         return results
