@@ -1,27 +1,34 @@
 # -*- coding: utf-8 -*-
+# BSD 3-Clause License
 #
-# Copyright (C) 2019 Chris Caron <lead2gold@gmail.com>
-# All rights reserved.
+# Apprise - Push Notification Library.
+# Copyright (c) 2023, Chris Caron <lead2gold@gmail.com>
 #
-# This code is licensed under the MIT License.
+# Redistribution and use in source and binary forms, with or without
+# modification, are permitted provided that the following conditions are met:
 #
-# Permission is hereby granted, free of charge, to any person obtaining a copy
-# of this software and associated documentation files(the "Software"), to deal
-# in the Software without restriction, including without limitation the rights
-# to use, copy, modify, merge, publish, distribute, sublicense, and / or sell
-# copies of the Software, and to permit persons to whom the Software is
-# furnished to do so, subject to the following conditions :
+# 1. Redistributions of source code must retain the above copyright notice,
+#    this list of conditions and the following disclaimer.
 #
-# The above copyright notice and this permission notice shall be included in
-# all copies or substantial portions of the Software.
+# 2. Redistributions in binary form must reproduce the above copyright notice,
+#    this list of conditions and the following disclaimer in the documentation
+#    and/or other materials provided with the distribution.
 #
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.IN NO EVENT SHALL THE
-# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
-# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
-# THE SOFTWARE.
+# 3. Neither the name of the copyright holder nor the names of its
+#    contributors may be used to endorse or promote products derived from
+#    this software without specific prior written permission.
+#
+# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
+# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+# POSSIBILITY OF SUCH DAMAGE.
 
 import dataclasses
 import re
@@ -36,6 +43,7 @@ from email import charset
 
 from socket import error as SocketError
 from datetime import datetime
+from datetime import timezone
 
 from .NotifyBase import NotifyBase
 from ..URLBase import PrivacyMode
@@ -63,15 +71,23 @@ class WebBaseLogin:
 
 # Secure Email Modes
 class SecureMailMode:
+    INSECURE = "insecure"
     SSL = "ssl"
     STARTTLS = "starttls"
 
 
 # Define all of the secure modes (used during validation)
-SECURE_MODES = (
-    SecureMailMode.SSL,
-    SecureMailMode.STARTTLS,
-)
+SECURE_MODES = {
+    SecureMailMode.STARTTLS: {
+        'default_port': 587,
+    },
+    SecureMailMode.SSL: {
+        'default_port': 465,
+    },
+    SecureMailMode.INSECURE: {
+        'default_port': 25,
+    },
+}
 
 # To attempt to make this script stupid proof, if we detect an email address
 # that is part of the this table, we can pre-use a lot more defaults if they
@@ -328,15 +344,6 @@ class NotifyEmail(NotifyBase):
     # Default Notify Format
     notify_format = NotifyFormat.HTML
 
-    # Default Non-Encryption Port
-    default_port = 25
-
-    # Default Secure Port
-    default_secure_port = 587
-
-    # Default Secure Mode
-    default_secure_mode = SecureMailMode.STARTTLS
-
     # Default SMTP Timeout (in seconds)
     socket_connect_timeout = 15
 
@@ -378,8 +385,13 @@ class NotifyEmail(NotifyBase):
             'min': 1,
             'max': 65535,
         },
+        'target_email': {
+            'name': _('Target Email'),
+            'type': 'string',
+            'map_to': 'targets',
+        },
         'targets': {
-            'name': _('Target Emails'),
+            'name': _('Targets'),
             'type': 'list:string',
         },
     })
@@ -446,14 +458,6 @@ class NotifyEmail(NotifyBase):
         """
         super().__init__(**kwargs)
 
-        # Handle SMTP vs SMTPS (Secure vs UnSecure)
-        if not self.port:
-            if self.secure:
-                self.port = self.default_secure_port
-
-            else:
-                self.port = self.default_port
-
         # Acquire Email 'To'
         self.targets = list()
 
@@ -511,9 +515,14 @@ class NotifyEmail(NotifyBase):
             smtp_host if isinstance(smtp_host, str) else ''
 
         # Now detect secure mode
-        self.secure_mode = self.default_secure_mode \
-            if not isinstance(secure_mode, str) \
-            else secure_mode.lower()
+        if secure_mode:
+            self.secure_mode = None \
+                if not isinstance(secure_mode, str) \
+                else secure_mode.lower()
+        else:
+            self.secure_mode = SecureMailMode.INSECURE \
+                if not self.secure else self.template_args['mode']['default']
+
         if self.secure_mode not in SECURE_MODES:
             msg = 'The secure mode specified ({}) is invalid.'\
                   .format(secure_mode)
@@ -590,6 +599,15 @@ class NotifyEmail(NotifyBase):
         # Apply any defaults based on certain known configurations
         self.NotifyEmailDefaults(secure_mode=secure_mode, **kwargs)
 
+        if not self.secure and self.secure_mode != SecureMailMode.INSECURE:
+            # Enable Secure mode if not otherwise set
+            self.secure = True
+
+        if not self.port:
+            # Assign our port based on our secure_mode if not otherwise
+            # detected
+            self.port = SECURE_MODES[self.secure_mode]['default_port']
+
         # if there is still no smtp_host then we fall back to the hostname
         if not self.smtp_host:
             self.smtp_host = self.host
@@ -653,11 +671,11 @@ class NotifyEmail(NotifyBase):
                 if login_type:
                     # only apply additional logic to our user if a login_type
                     # was specified.
-                    if is_email(self.user) and \
-                       WebBaseLogin.EMAIL not in login_type:
-                        # Email specified but login type
-                        # not supported; switch it to user id
-                        self.user = match.group('id')
+                    if is_email(self.user):
+                        if WebBaseLogin.EMAIL not in login_type:
+                            # Email specified but login type
+                            # not supported; switch it to user id
+                            self.user = match.group('id')
 
                     elif WebBaseLogin.USERID not in login_type:
                         # user specified but login type
@@ -793,7 +811,8 @@ class NotifyEmail(NotifyBase):
             base['To'] = formataddr((to_name, to_addr), charset='utf-8')
             base['Message-ID'] = make_msgid(domain=self.smtp_host)
             base['Date'] = \
-                datetime.utcnow().strftime("%a, %d %b %Y %H:%M:%S +0000")
+                datetime.now(timezone.utc)\
+                .strftime("%a, %d %b %Y %H:%M:%S +0000")
             base['X-Application'] = self.app_id
 
             if cc:
@@ -824,7 +843,7 @@ class NotifyEmail(NotifyBase):
         try:
             self.logger.debug('Connecting to remote SMTP server...')
             socket_func = smtplib.SMTP
-            if self.secure and self.secure_mode == SecureMailMode.SSL:
+            if self.secure_mode == SecureMailMode.SSL:
                 self.logger.debug('Securing connection with SSL...')
                 socket_func = smtplib.SMTP_SSL
 
@@ -835,7 +854,7 @@ class NotifyEmail(NotifyBase):
                 timeout=self.socket_connect_timeout,
             )
 
-            if self.secure and self.secure_mode == SecureMailMode.STARTTLS:
+            if self.secure_mode == SecureMailMode.STARTTLS:
                 # Handle Secure Connections
                 self.logger.debug('Securing connection with STARTTLS...')
                 socket.starttls()
@@ -965,8 +984,7 @@ class NotifyEmail(NotifyBase):
             )
 
         # Default Port setup
-        default_port = \
-            self.default_secure_port if self.secure else self.default_port
+        default_port = SECURE_MODES[self.secure_mode]['default_port']
 
         # a simple boolean check as to whether we display our target emails
         # or not
@@ -988,6 +1006,13 @@ class NotifyEmail(NotifyBase):
             params=NotifyEmail.urlencode(params),
         )
 
+    def __len__(self):
+        """
+        Returns the number of targets associated with this notification
+        """
+        targets = len(self.targets)
+        return targets if targets > 0 else 1
+
     @staticmethod
     def parse_url(url):
         """
