@@ -1,27 +1,34 @@
 # -*- coding: utf-8 -*-
+# BSD 3-Clause License
 #
-# Copyright (C) 2019 Chris Caron <lead2gold@gmail.com>
-# All rights reserved.
+# Apprise - Push Notification Library.
+# Copyright (c) 2023, Chris Caron <lead2gold@gmail.com>
 #
-# This code is licensed under the MIT License.
+# Redistribution and use in source and binary forms, with or without
+# modification, are permitted provided that the following conditions are met:
 #
-# Permission is hereby granted, free of charge, to any person obtaining a copy
-# of this software and associated documentation files(the "Software"), to deal
-# in the Software without restriction, including without limitation the rights
-# to use, copy, modify, merge, publish, distribute, sublicense, and / or sell
-# copies of the Software, and to permit persons to whom the Software is
-# furnished to do so, subject to the following conditions :
+# 1. Redistributions of source code must retain the above copyright notice,
+#    this list of conditions and the following disclaimer.
 #
-# The above copyright notice and this permission notice shall be included in
-# all copies or substantial portions of the Software.
+# 2. Redistributions in binary form must reproduce the above copyright notice,
+#    this list of conditions and the following disclaimer in the documentation
+#    and/or other materials provided with the distribution.
 #
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.IN NO EVENT SHALL THE
-# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
-# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
-# THE SOFTWARE.
+# 3. Neither the name of the copyright holder nor the names of its
+#    contributors may be used to endorse or promote products derived from
+#    this software without specific prior written permission.
+#
+# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
+# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+# POSSIBILITY OF SUCH DAMAGE.
 
 import os
 import re
@@ -31,10 +38,7 @@ from os.path import dirname
 from os.path import abspath
 
 # Used for testing
-from . import NotifyEmail as NotifyEmailBase
-
-# NotifyBase object is passed in as a module not class
-from . import NotifyBase
+from .NotifyBase import NotifyBase
 
 from ..common import NotifyImageSize
 from ..common import NOTIFY_IMAGE_SIZES
@@ -53,9 +57,6 @@ __all__ = [
     'NotifyImageSize', 'NOTIFY_IMAGE_SIZES', 'NotifyType', 'NOTIFY_TYPES',
     'NotifyBase',
 
-    # NotifyEmail Base Module (used for NotifyEmail testing)
-    'NotifyEmailBase',
-
     # Tokenizer
     'url_to_dict',
 ]
@@ -164,6 +165,9 @@ def _sanitize_token(tokens, default_delimiter):
 
     """
 
+    # Used for tracking groups
+    group_map = {}
+
     # Iterate over our tokens
     for key in tokens.keys():
 
@@ -180,14 +184,27 @@ def _sanitize_token(tokens, default_delimiter):
             # Default type to key
             tokens[key]['map_to'] = key
 
+        # Track our map_to objects
+        if tokens[key]['map_to'] not in group_map:
+            group_map[tokens[key]['map_to']] = set()
+        group_map[tokens[key]['map_to']].add(key)
+
         if 'type' not in tokens[key]:
             # Default type to string
             tokens[key]['type'] = 'string'
 
-        elif tokens[key]['type'].startswith('list') \
-                and 'delim' not in tokens[key]:
-            # Default list delimiter (if not otherwise specified)
-            tokens[key]['delim'] = default_delimiter
+        elif tokens[key]['type'].startswith('list'):
+            if 'delim' not in tokens[key]:
+                # Default list delimiter (if not otherwise specified)
+                tokens[key]['delim'] = default_delimiter
+
+            if key in group_map[tokens[key]['map_to']]:  # pragma: no branch
+                # Remove ourselves from the list
+                group_map[tokens[key]['map_to']].remove(key)
+
+            # Pointing to the set directly so we can dynamically update
+            # ourselves
+            tokens[key]['group'] = group_map[tokens[key]['map_to']]
 
         elif tokens[key]['type'].startswith('choice') \
                 and 'default' not in tokens[key] \
@@ -265,6 +282,13 @@ def details(plugin):
     #            # Identifies if the entry specified is required or not
     #            'required': True,
     #
+    #            # Identifies all tokens detected to be associated with the
+    #            # list:string
+    #            # This is ony present in list:string objects and is only set
+    #            # if this element acts as an alias for several other
+    #            # kwargs/fields.
+    #            'group': [],
+    #
     #            # Identify a default value
     #            'default': 'http',
     #
