@@ -1,27 +1,35 @@
 # -*- coding: utf-8 -*-
+# BSD 3-Clause License
 #
-# Copyright (C) 2021 Chris Caron <lead2gold@gmail.com>
-# All rights reserved.
+# Apprise - Push Notification Library.
+# Copyright (c) 2023, Chris Caron <lead2gold@gmail.com>
 #
-# This code is licensed under the MIT License.
+# Redistribution and use in source and binary forms, with or without
+# modification, are permitted provided that the following conditions are met:
 #
-# Permission is hereby granted, free of charge, to any person obtaining a copy
-# of this software and associated documentation files(the "Software"), to deal
-# in the Software without restriction, including without limitation the rights
-# to use, copy, modify, merge, publish, distribute, sublicense, and / or sell
-# copies of the Software, and to permit persons to whom the Software is
-# furnished to do so, subject to the following conditions :
+# 1. Redistributions of source code must retain the above copyright notice,
+#    this list of conditions and the following disclaimer.
 #
-# The above copyright notice and this permission notice shall be included in
-# all copies or substantial portions of the Software.
+# 2. Redistributions in binary form must reproduce the above copyright notice,
+#    this list of conditions and the following disclaimer in the documentation
+#    and/or other materials provided with the distribution.
 #
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.IN NO EVENT SHALL THE
-# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
-# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
-# THE SOFTWARE.
+# 3. Neither the name of the copyright holder nor the names of its
+#    contributors may be used to endorse or promote products derived from
+#    this software without specific prior written permission.
+#
+# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
+# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+# POSSIBILITY OF SUCH DAMAGE.
+
 import re
 import os
 import requests
@@ -122,9 +130,6 @@ class AppriseURLTester:
         # Our expected server objects
         _self = meta.get('self', None)
 
-        # Our expected Query response (True, False, or exception type)
-        response = meta.get('response', True)
-
         # Our expected privacy url
         # Don't set this if don't need to check it's value
         privacy_url = meta.get('privacy_url')
@@ -132,20 +137,6 @@ class AppriseURLTester:
         # Our regular expression
         url_matches = meta.get('url_matches')
 
-        # Allow us to force the server response code to be something other then
-        # the defaults
-        requests_response_code = meta.get(
-            'requests_response_code',
-            requests.codes.ok if response else requests.codes.not_found,
-        )
-
-        # Allow us to force the server response text to be something other then
-        # the defaults
-        requests_response_text = meta.get('requests_response_text')
-        if not isinstance(requests_response_text, str):
-            # Convert to string
-            requests_response_text = dumps(requests_response_text)
-
         # Whether or not we should include an image with our request; unless
         # otherwise specified, we assume that images are to be included
         include_image = meta.get('include_image', True)
@@ -158,35 +149,12 @@ class AppriseURLTester:
             asset = AppriseAsset(image_path_mask=False, image_url_mask=False)
             asset.image_url_logo = None
 
-        test_requests_exceptions = meta.get(
-            'test_requests_exceptions', False)
-
         # Mock our request object
         robj = mock.Mock()
         robj.content = u''
         mock_get.return_value = robj
         mock_post.return_value = robj
 
-        if test_requests_exceptions is False:
-            # Handle our default response
-            mock_post.return_value.status_code = requests_response_code
-            mock_get.return_value.status_code = requests_response_code
-
-            # Handle our default text response
-            mock_get.return_value.content = requests_response_text
-            mock_post.return_value.content = requests_response_text
-            mock_get.return_value.text = requests_response_text
-            mock_post.return_value.text = requests_response_text
-
-            # Ensure there is no side effect set
-            mock_post.side_effect = None
-            mock_get.side_effect = None
-
-        else:
-            # Handle exception testing; first we turn the boolean flag
-            # into a list of exceptions
-            test_requests_exceptions = self.req_exceptions
-
         try:
             # We can now instantiate our object:
             obj = Apprise.instantiate(
@@ -235,6 +203,9 @@ class AppriseURLTester:
             # this url
             assert isinstance(obj.url(), str) is True
 
+            # Verify we can acquire a target count as an integer
+            assert isinstance(len(obj), int)
+
             # Test url() with privacy=True
             assert isinstance(
                 obj.url(privacy=True), str) is True
@@ -272,6 +243,14 @@ class AppriseURLTester:
                     url, obj.url()))
                 assert False
 
+            # Verify there is no change from the old and the new
+            if len(obj) != len(obj_cmp):
+                print('%d targets found in %s' % (
+                    len(obj), obj.url(privacy=True)))
+                print('But %d targets found in %s' % (
+                    len(obj_cmp), obj_cmp.url(privacy=True)))
+                raise AssertionError("Target miscount %d != %d")
+
             # Tidy our object
             del obj_cmp
 
@@ -300,8 +279,9 @@ class AppriseURLTester:
     @mock.patch('requests.head')
     @mock.patch('requests.put')
     @mock.patch('requests.delete')
-    def __notify(self, url, obj, meta, asset, mock_del, mock_put, mock_head,
-                 mock_post, mock_get):
+    @mock.patch('requests.patch')
+    def __notify(self, url, obj, meta, asset, mock_patch, mock_del, mock_put,
+                 mock_head, mock_post, mock_get):
         """
         Perform notification testing against object specified
         """
@@ -338,9 +318,19 @@ class AppriseURLTester:
         # Allow us to force the server response text to be something other then
         # the defaults
         requests_response_text = meta.get('requests_response_text')
-        if not isinstance(requests_response_text, str):
+        requests_response_content = None
+
+        if isinstance(requests_response_text, str):
+            requests_response_content = requests_response_text.encode('utf-8')
+
+        elif isinstance(requests_response_text, bytes):
+            requests_response_content = requests_response_text
+            requests_response_text = requests_response_text.decode('utf-8')
+
+        elif not isinstance(requests_response_text, str):
             # Convert to string
             requests_response_text = dumps(requests_response_text)
+            requests_response_content = requests_response_text.encode('utf-8')
 
         # A request
         robj = mock.Mock()
@@ -348,6 +338,7 @@ class AppriseURLTester:
         mock_get.return_value = robj
         mock_post.return_value = robj
         mock_head.return_value = robj
+        mock_patch.return_value = robj
         mock_del.return_value = robj
         mock_put.return_value = robj
 
@@ -358,19 +349,22 @@ class AppriseURLTester:
             mock_del.return_value.status_code = requests_response_code
             mock_post.return_value.status_code = requests_response_code
             mock_get.return_value.status_code = requests_response_code
+            mock_patch.return_value.status_code = requests_response_code
 
             # Handle our default text response
-            mock_get.return_value.content = requests_response_text
-            mock_post.return_value.content = requests_response_text
-            mock_del.return_value.content = requests_response_text
-            mock_put.return_value.content = requests_response_text
-            mock_head.return_value.content = requests_response_text
+            mock_get.return_value.content = requests_response_content
+            mock_post.return_value.content = requests_response_content
+            mock_del.return_value.content = requests_response_content
+            mock_put.return_value.content = requests_response_content
+            mock_head.return_value.content = requests_response_content
+            mock_patch.return_value.content = requests_response_content
 
             mock_get.return_value.text = requests_response_text
             mock_post.return_value.text = requests_response_text
             mock_put.return_value.text = requests_response_text
             mock_del.return_value.text = requests_response_text
             mock_head.return_value.text = requests_response_text
+            mock_patch.return_value.text = requests_response_text
 
             # Ensure there is no side effect set
             mock_post.side_effect = None
@@ -378,6 +372,7 @@ class AppriseURLTester:
             mock_put.side_effect = None
             mock_head.side_effect = None
             mock_get.side_effect = None
+            mock_patch.side_effect = None
 
         else:
             # Handle exception testing; first we turn the boolean flag
@@ -387,11 +382,19 @@ class AppriseURLTester:
         try:
             if test_requests_exceptions is False:
 
+                # Verify we can acquire a target count as an integer
+                targets = len(obj)
+
                 # check that we're as expected
                 assert obj.notify(
                     body=self.body, title=self.title,
                     notify_type=notify_type) == notify_response
 
+                if notify_response:
+                    # If we successfully got a response, there must have been
+                    # at least 1 target present
+                    assert targets > 0
+
                 # check that this doesn't change using different overflow
                 # methods
                 assert obj.notify(
@@ -463,6 +466,7 @@ class AppriseURLTester:
                         os.path.join(self.__test_var_dir, 'apprise-test.png'),
                         os.path.join(self.__test_var_dir, 'apprise-test.jpeg'),
                     ))
+
                     assert obj.notify(
                         body=self.body, title=self.title,
                         notify_type=notify_type,
@@ -476,6 +480,7 @@ class AppriseURLTester:
                     mock_del.side_effect = _exception
                     mock_put.side_effect = _exception
                     mock_get.side_effect = _exception
+                    mock_patch.side_effect = _exception
 
                     try:
                         assert obj.notify(
@@ -520,6 +525,7 @@ class AppriseURLTester:
                     mock_put.side_effect = _exception
                     mock_head.side_effect = _exception
                     mock_get.side_effect = _exception
+                    mock_patch.side_effect = _exception
 
                     try:
                         assert obj.notify(
