@@ -1,29 +1,38 @@
 # -*- coding: utf-8 -*-
+# BSD 3-Clause License
 #
-# Copyright (C) 2019 Chris Caron <lead2gold@gmail.com>
-# All rights reserved.
+# Apprise - Push Notification Library.
+# Copyright (c) 2023, Chris Caron <lead2gold@gmail.com>
 #
-# This code is licensed under the MIT License.
+# Redistribution and use in source and binary forms, with or without
+# modification, are permitted provided that the following conditions are met:
 #
-# Permission is hereby granted, free of charge, to any person obtaining a copy
-# of this software and associated documentation files(the "Software"), to deal
-# in the Software without restriction, including without limitation the rights
-# to use, copy, modify, merge, publish, distribute, sublicense, and / or sell
-# copies of the Software, and to permit persons to whom the Software is
-# furnished to do so, subject to the following conditions :
+# 1. Redistributions of source code must retain the above copyright notice,
+#    this list of conditions and the following disclaimer.
 #
-# The above copyright notice and this permission notice shall be included in
-# all copies or substantial portions of the Software.
+# 2. Redistributions in binary form must reproduce the above copyright notice,
+#    this list of conditions and the following disclaimer in the documentation
+#    and/or other materials provided with the distribution.
 #
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.IN NO EVENT SHALL THE
-# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
-# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
-# THE SOFTWARE.
+# 3. Neither the name of the copyright holder nor the names of its
+#    contributors may be used to endorse or promote products derived from
+#    this software without specific prior written permission.
+#
+# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
+# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+# POSSIBILITY OF SUCH DAMAGE.
 
 from __future__ import print_function
+import asyncio
+import concurrent.futures
 import re
 import sys
 import pytest
@@ -49,12 +58,9 @@ from apprise import common
 from apprise.plugins import __load_matrix
 from apprise.plugins import __reset_matrix
 from apprise.utils import parse_list
+from helpers import OuterEventLoop
 import inspect
 
-# Sending notifications requires the coroutines to be awaited, so we need to
-# wrap the original function when mocking it.
-import apprise.py3compat.asyncio as py3aio
-
 # Disable logging for a cleaner testing output
 import logging
 logging.disable(logging.CRITICAL)
@@ -79,10 +85,12 @@ def test_apprise_async():
     API: Apprise() object asynchronous methods
 
     """
-    def do_notify(server, *args, **kwargs):
-        return py3aio.tosync(server.async_notify(*args, **kwargs))
+    with OuterEventLoop() as loop:
+        def do_notify(server, *args, **kwargs):
+            return loop.run_until_complete(
+                server.async_notify(*args, **kwargs))
 
-    apprise_test(do_notify)
+        apprise_test(do_notify)
 
 
 def apprise_test(do_notify):
@@ -292,6 +300,10 @@ def apprise_test(do_notify):
             # Pretend everything is okay
             raise TypeError()
 
+        async def async_notify(self, **kwargs):
+            # Pretend everything is okay (async)
+            raise TypeError()
+
         def url(self, **kwargs):
             # Support URL
             return ''
@@ -301,6 +313,10 @@ def apprise_test(do_notify):
             # Pretend everything is okay
             raise RuntimeError()
 
+        async def async_notify(self, **kwargs):
+            # Pretend everything is okay (async)
+            raise TypeError()
+
         def url(self, **kwargs):
             # Support URL
             return ''
@@ -311,6 +327,10 @@ def apprise_test(do_notify):
             # Pretend everything is okay
             return False
 
+        async def async_notify(self, **kwargs):
+            # Pretend everything is okay (async)
+            raise TypeError()
+
         def url(self, **kwargs):
             # Support URL
             return ''
@@ -539,10 +559,12 @@ def test_apprise_tagging_async(mock_post, mock_get):
     API: Apprise() object tagging functionality asynchronous methods
 
     """
-    def do_notify(server, *args, **kwargs):
-        return py3aio.tosync(server.async_notify(*args, **kwargs))
+    with OuterEventLoop() as loop:
+        def do_notify(server, *args, **kwargs):
+            return loop.run_until_complete(
+                server.async_notify(*args, **kwargs))
 
-    apprise_tagging_test(mock_post, mock_get, do_notify)
+        apprise_tagging_test(mock_post, mock_get, do_notify)
 
 
 def apprise_tagging_test(mock_post, mock_get, do_notify):
@@ -726,7 +748,10 @@ def test_apprise_notify_formats(tmpdir):
     # other if/else parts of the code that aren't otherwise called
     __load_matrix()
 
-    a = Apprise()
+    # Need to set async_mode=False to call notify() instead of async_notify().
+    asset = AppriseAsset(async_mode=False)
+
+    a = Apprise(asset=asset)
 
     # no items
     assert len(a) == 0
@@ -736,7 +761,7 @@ def test_apprise_notify_formats(tmpdir):
         notify_format = NotifyFormat.TEXT
 
         def __init__(self, **kwargs):
-            super().__init__()
+            super().__init__(**kwargs)
 
         def notify(self, **kwargs):
             # Pretend everything is okay
@@ -752,7 +777,7 @@ def test_apprise_notify_formats(tmpdir):
         notify_format = NotifyFormat.HTML
 
         def __init__(self, **kwargs):
-            super().__init__()
+            super().__init__(**kwargs)
 
         def notify(self, **kwargs):
             # Pretend everything is okay
@@ -768,7 +793,7 @@ def test_apprise_notify_formats(tmpdir):
         notify_format = NotifyFormat.MARKDOWN
 
         def __init__(self, **kwargs):
-            super().__init__()
+            super().__init__(**kwargs)
 
         def notify(self, **kwargs):
             # Pretend everything is okay
@@ -1461,6 +1486,7 @@ def test_apprise_details_plugin_verification():
     valid_schema_keys = (
         'name', 'private', 'required', 'type', 'values', 'min', 'max',
         'regex', 'default', 'list', 'delim', 'prefix', 'map_to', 'alias_of',
+        'group',
     )
     for entry in details['schemas']:
 
@@ -1756,8 +1782,10 @@ def test_apprise_details_plugin_verification():
 
 
 @mock.patch('requests.post')
-@mock.patch('apprise.py3compat.asyncio.notify', wraps=py3aio.notify)
-def test_apprise_async_mode(mock_async_notify, mock_post, tmpdir):
+@mock.patch('asyncio.gather', wraps=asyncio.gather)
+@mock.patch('concurrent.futures.ThreadPoolExecutor',
+            wraps=concurrent.futures.ThreadPoolExecutor)
+def test_apprise_async_mode(mock_threadpool, mock_gather, mock_post, tmpdir):
     """
     API: Apprise() async_mode tests
 
@@ -1790,9 +1818,9 @@ def test_apprise_async_mode(mock_async_notify, mock_post, tmpdir):
     # Send Notifications Asyncronously
     assert a.notify("async") is True
 
-    # Verify our async code got executed
-    assert mock_async_notify.call_count == 1
-    mock_async_notify.reset_mock()
+    # Verify our thread pool was created
+    assert mock_threadpool.call_count == 1
+    mock_threadpool.reset_mock()
 
     # Provide an over-ride now
     asset = AppriseAsset(async_mode=False)
@@ -1816,9 +1844,9 @@ def test_apprise_async_mode(mock_async_notify, mock_post, tmpdir):
 
     # Send Notifications Syncronously
     assert a.notify("sync") is True
-    # Verify our async code got called
-    assert mock_async_notify.call_count == 1
-    mock_async_notify.reset_mock()
+    # Sequential send doesn't require a gather
+    assert mock_gather.call_count == 0
+    mock_gather.reset_mock()
 
     # another way of looking a our false set asset configuration
     assert a[0].asset.async_mode is False
@@ -1839,9 +1867,9 @@ def test_apprise_async_mode(mock_async_notify, mock_post, tmpdir):
     # Send 1 Notification Syncronously, the other Asyncronously
     assert a.notify("a mixed batch") is True
 
-    # Verify our async code got called
-    assert mock_async_notify.call_count == 1
-    mock_async_notify.reset_mock()
+    # Verify we didn't use a thread pool for a single notification
+    assert mock_threadpool.call_count == 0
+    mock_threadpool.reset_mock()
 
 
 def test_notify_matrix_dynamic_importing(tmpdir):
