@@ -1,27 +1,35 @@
 # -*- coding: utf-8 -*-
+# BSD 3-Clause License
 #
-# Copyright (C) 2021 Chris Caron <lead2gold@gmail.com>
-# All rights reserved.
+# Apprise - Push Notification Library.
+# Copyright (c) 2023, Chris Caron <lead2gold@gmail.com>
 #
-# This code is licensed under the MIT License.
+# Redistribution and use in source and binary forms, with or without
+# modification, are permitted provided that the following conditions are met:
 #
-# Permission is hereby granted, free of charge, to any person obtaining a copy
-# of this software and associated documentation files(the "Software"), to deal
-# in the Software without restriction, including without limitation the rights
-# to use, copy, modify, merge, publish, distribute, sublicense, and / or sell
-# copies of the Software, and to permit persons to whom the Software is
-# furnished to do so, subject to the following conditions :
+# 1. Redistributions of source code must retain the above copyright notice,
+#    this list of conditions and the following disclaimer.
 #
-# The above copyright notice and this permission notice shall be included in
-# all copies or substantial portions of the Software.
+# 2. Redistributions in binary form must reproduce the above copyright notice,
+#    this list of conditions and the following disclaimer in the documentation
+#    and/or other materials provided with the distribution.
 #
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.IN NO EVENT SHALL THE
-# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
-# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
-# THE SOFTWARE.
+# 3. Neither the name of the copyright holder nor the names of its
+#    contributors may be used to endorse or promote products derived from
+#    this software without specific prior written permission.
+#
+# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
+# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+# POSSIBILITY OF SUCH DAMAGE.
+
 import os
 from json import loads, dumps
 from unittest import mock
@@ -291,8 +299,8 @@ def test_plugin_smseagle_edge_cases(mock_post):
     aobj = Apprise()
     assert aobj.add(
         "smseagles://token@localhost:231/{}".format(target))
+    assert len(aobj) == 1
     assert aobj.notify(title=title, body=body)
-
     assert mock_post.call_count == 1
 
     details = mock_post.call_args_list[0]
@@ -307,8 +315,8 @@ def test_plugin_smseagle_edge_cases(mock_post):
     assert aobj.add(
         "smseagles://token@localhost:231/{}?status=Yes".format(
             target))
+    assert len(aobj) == 1
     assert aobj.notify(title=title, body=body)
-
     assert mock_post.call_count == 1
 
     details = mock_post.call_args_list[0]
@@ -340,6 +348,8 @@ def test_plugin_smseagle_result_set(mock_post):
     aobj.add(
         'smseagle://token@10.0.0.112:8080/+12512222222/+12513333333/'
         '12514444444?batch=yes')
+    # In a batch mode we can shove them all into 1 call
+    assert len(aobj[0]) == 1
 
     assert aobj.notify(title=title, body=body)
 
@@ -378,6 +388,7 @@ def test_plugin_smseagle_result_set(mock_post):
     aobj.add(
         'smseagle://token@10.0.0.112:8080/#group/Contact/'
         '123456789?batch=no')
+    assert len(aobj[0]) == 3
 
     assert aobj.notify(title=title, body=body)
 
@@ -455,6 +466,8 @@ def test_plugin_smseagle_result_set(mock_post):
         'smseagle://token@10.0.0.112:8080/513333333/#group1/@contact1/'
         'contact2/12514444444?batch=yes')
 
+    # contacts and numbers can be combined and is calculated in batch response
+    assert len(aobj[0]) == 3
     assert aobj.notify(title=title, body=body)
 
     # There is a unique post to each (group, contact x2, and phone x2)
